/****************************************************************************
** File:    mail_utl.c
**
** Description: This module contains a function for sending mail using
**              the /bin/mailx mail program. This variant of the
**              send_mail function demonstrates how NULL parameters can
**              be replaced by "defaults" defined in a C shared library.
**              It also demonstrations how programmer-defined exceptions
**              can be raised.
**
**      Author: Fred Polizo, Jr.    FP Systems
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <oci.h>

/* Tested with Solaris 2.5.1 and Oracle 8.0.3 */

/****************************************************************************
** Function:    send_mail
**
** Description: A function that sends a mail message to the specified
**              list of recipients (space or comma delimited).
**
**              Returns a value of 0 if no errors were reported by the OS.
**              Otherwise, it returns a OS error code.
****************************************************************************/
sb4 
send_mail (OCIExtProcContext *pWithContext,
           char     *pReturnAddress,
           sb2      indReturnAddress,
           char     *pRecipientList,
           sb2      indRecipientList,
           char     *pSubject,
           sb2      indSubject,
           char     *pMessage,
           sb2      indMessage)
{
    char mailCmd[BUFSIZ];
    char returnAddrOption[BUFSIZ];
    sb4 errorStatus;
    FILE *pMailPipe;

    /* If there are NULL input parameters, try to "fill in the blanks"
    || and mail the message anyway.
    */

    if (indRecipientList == OCI_IND_NULL)
    {
        /* Raise an unnamed programmer-defined exception since a
        || recipients list is required to send a mail message. 
        */
        if (OCIExtProcRaiseExcpWithMsg (pWithContext,
                                        (int)20101, 
                                        "a recipient list is required",
                                        0) == OCIEXTPROC_SUCCESS)
        {
            /* Return immediately */
            return -1;
        }
        else
        {
            /* Error raising exception, abort this 'extproc' process. */
            abort();
        } 
    }

    /* Test for null arguments and supply defaults where appropriate */
    if (indMessage == OCI_IND_NULL)
    {
        pMessage = "<No Message>";
    }
    
    if (indSubject == OCI_IND_NULL)
    {
        pSubject = "<No Subject Given>";
    }

    if (indReturnAddress == OCI_IND_NULL)
    {
        /* No return address option requested */
        returnAddrOption[0] = '\0';
    }
    else
    {
        sprintf(returnAddrOption,
                "-r '%s'",
                pReturnAddress);
    }

    /* Now format the mail command line */
    sprintf(mailCmd,
            "/bin/mailx %s -s '%s' %s",
            returnAddrOption,
            pSubject,
            pRecipientList);

    /* Reset the last system error value */
    errno = 0;
    
    /* Open a pipe to the mail program */
    if ( (pMailPipe = popen (mailCmd, "w")) != NULL )
    {
        if ( fwrite (pMessage, strlen (pMessage), 1, pMailPipe)
             == strlen (pMessage) )
        {
            /* Now close the pipe and return the termination status */
            errorStatus = pclose(pMailPipe);
        }
        else
        {
            /* We couldn't write the message buffer, return the error */
            errorStatus = errno;
            (void) pclose (pMailPipe);
        }
    }
    else
    {
        /* We failed to open a pipe to the command, return the error */
        errorStatus = errno;
    }

    return errorStatus;
}
